/*******************************************************************************

  JOGO_DA_VELHA_LINEAR

  C/ PIC 12f675 e LCD 16X2 (usando LPLEX a 3 fios)


  Autor: Cludio Lrios

  Data:  08/10/2017

  Verso do famoso "Jogo da Velha" com apresentao
  das 9 casas em apenas uma nica linha do LCD
  Dispem de 1 interruptor (CURSOR) para deslocar (qual casa ser preenchida) e
  mais 2 interruptores, sendo um para o jogador que usa  o "X" (J1) e outro
  para o jogador (J2) que usa o "O".
  Na segunda linha ir acumular os pontos de cada jogador. Desligue e
  ligue a alimentao para resetar todos os valores.
  Para novo jogo aperte CURSOR. Ser mantido o placar anterior.
  A seta (>) na frente de J1 ou J2 indica de que  a vez de jogar.
  A letra 'E' entre J1 e J2 aparece quando o jogo resulta em empate.
  Pontuao:
  Vitria Simples=3 pontos
  Vitria Dupla = 6 pontos
  Empate= 1 ponto para cada jogador
  Contagem mxima permitida=99
  
  Uso didtico

  Este arquivo  parte integrante do blog PicSource (Lrios.Tecnologia.WS)

*******************************************************************************/
#include <12F675.h>
#device adc=10  //10 bits (no usado)
#use delay(clock=4000000)//frequencia 4 mhz osc.interno
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, PUT, BROWNOUT//configurao

#rom 0x2100 = {" Jogo da Velha  ","Blog PicSource"}// EEprom load
//bytes
#byte  trisio = 0x85
#byte  gpio   = 0x05
#byte  tmr0   = 0x01

//pinos do lcd (metodo Lariosplex 3 FIOS)
#bit DB76     = 0x5.5  //pino 2 db7=direto db6=resistor 10k +cap 10nF ao gnd
#bit DB54     = 0x5.4  //pino 3 db5=direto db4=resistor 10k +cap 10nF ao gnd
#bit E_RS     = 0x5.2  //pino 5
#bit j1       = 0x5.0  //pino 7
#bit j2       = 0x5.1  //pino 6
#bit cursor   = 0x05.3  //pino 4 sw
#bit wpu0     = 0x95.0
#bit wpu1     = 0x95.1
#bit gppu     = 0x81.7

//defines para LPLEX
#define vcarga 500 // por tentativa e erro (entre 200 a 500)
#define vdescarga 500//idem
#define v_delay_clk 2  // de 2 a 50 conforme LCD

//variveis globais
unsigned int1 RS,vez,tr1;// flags
unsigned int8 a,b,pos,cnt,res,cont=0,win=0;
unsigned int8 jogo[16];//armazena caracteres para linha 1
unsigned int8 jogador[]={0,0,0,0};//pontuao na linha 2
/*******************************************************************************
                          SUBROTINAS
*******************************************************************************/
void descarregar(){ DB76=0;DB54=0;E_RS=0; delay_us(vdescarga);}
void t_carga(){delay_us(vcarga);}
void clk(){ E_RS=1; delay_cycles(v_delay_clk); E_RS=0;}//gera clock par Lcd

/*******************************************************************************
                          SUBROTINAS de LCD
*******************************************************************************/


/*******************************************************************************
                         ENVIA NIBBLE
*******************************************************************************/
void env_nible(unsigned int8 nib){
    descarregar();//descarrega capacitores
    nib&=0x0f;//limita
    if(nib&0x01)DB54=1;//testa bit 0
    if(nib&0x04)DB76=1;//testa bit 2
    if(RS)E_RS=1;else E_RS=0;//comando ou dado
    t_carga();
    if(nib&0x02)DB54=1; else DB54=0;//testa bit 1
    if(nib&0x08)DB76=1; else DB76=0;//testa bit 3
    clk();//clock
}
/*******************************************************************************
                          ENVIA DADOS PARA O LCD
*******************************************************************************/
void env_lcd_d(unsigned int8 dado){
 RS=1;//indica que enviar 'dados'
 env_nible(dado>>4);//high nibble primeiro
 env_nible(dado);
 delay_ms(4);
}
/*******************************************************************************
                         ENVIA COMANDOS PARA LCD
*******************************************************************************/
void env_lcd_c(unsigned int8 comando){
 RS=0;//indica que enviar 'comandos'
 env_nible(comando>>4);//high nibble primeiro
 env_nible(comando);
 delay_ms(4);
}
/*******************************************************************************
                         INICIALIZAO DO LCD
*******************************************************************************/
void Lcd_Init(){
  unsigned int8 a;
   descarregar();//descarrega capacitores
   RS=0; //enviar comando
   delay_ms(40);//delay inicial
   for(a=0;a<3;a++) { env_nible(3);delay_ms(5);}//envia 3x 0x03
   env_nible(2);//envia 0x02
   delay_ms(5);//delay 5 ms
   env_lcd_c(0x28);//ajusta para 4 bits o modo de envio
   env_lcd_c(0x06);// no shift
   env_lcd_c(0x0c);//Display ON, Cursor OFF, Blink OFF
  // env_lcd_c(0x0f);//Display ON, Cursor OFF, Blink On
   env_lcd_c(0x01);//limpa tela

}

/*******************************************************************************
                         DELAY
*******************************************************************************/
void delay_200ms(){ delay_ms(200);}
void delay_dbc(){delay_ms(20);}
void delay_1seg(){
   for(a=0;a<5;a++){//mantem a tela apagada por um segundo
    delay_200ms();
   }
}

/*******************************************************************************
                         ZERA PLACAR
*******************************************************************************/

void nova_partida(){

   env_lcd_c(0x01);//limpa tela
   delay_1seg();

   jogo[0]=' ';  //carrega buffer do jogo
   jogo[1]=' ';
   jogo[2]='_';
   jogo[3]='_';
   jogo[4]='_';
   jogo[5]=' ';
   jogo[6]='_';
   jogo[7]='_';
   jogo[8]='_';
   jogo[9]=' ';
   jogo[10]='_';
   jogo[11]='_';
   jogo[12]='_';
   jogo[13]=' ';
   jogo[14]=' ';
   jogo[15]=' ';

   cont=0;//zera contador de identificao de empate

   env_lcd_c(0x80);//incio da primeira linha

   for(b=0;b<16;b++){   //coloca os traos e espaos no inicio do jogo
   env_lcd_d(jogo[b]);

   }
   env_lcd_c(0xc0);  //zera placares
   if(!vez)env_lcd_d('>'); else env_lcd_d(' ');//mostra qual jogador ir jogar
   env_lcd_d('J');//mostra linha 2 com a pontuao
   env_lcd_d('1');
   env_lcd_d('=');
   env_lcd_d(jogador[0]|0x30);
   env_lcd_d(jogador[1]|0x30);
   env_lcd_c(0xc9);
   if(vez)env_lcd_d('>'); else env_lcd_d(' ');
   env_lcd_d('J');
   env_lcd_d('2');
   env_lcd_d('=');
   env_lcd_d(jogador[2]|0x30);
   env_lcd_d(jogador[3]|0x30);
   env_lcd_c(0x80);
   win=0;
}

/*******************************************************************************
         ROTINAS PARA PISCAR 'X' OU '0'
*******************************************************************************/
void pisca_pos_h(unsigned int8 posicao, unsigned int8 j){//NA HORIZONTAL
           env_lcd_c(0x0c);//desliga cursor
           for(a=0;a<10;a++){
             env_lcd_c(posicao);
             env_lcd_d(' ');
             env_lcd_d(' ');
             env_lcd_d(' ');
             delay_200ms();
             env_lcd_c(posicao);
             env_lcd_d(j);
             env_lcd_d(j);
             env_lcd_d(j);
             delay_200ms();
            }
}
void pisca_pos_v(unsigned int8 posicao, unsigned int8 j){//NA VERTICAL
           env_lcd_c(0x0c);//desliga cursor
           for(a=0;a<10;a++){
             env_lcd_c(posicao);
             env_lcd_d(' ');
             env_lcd_c(posicao+4);
             env_lcd_d(' ');
             env_lcd_c(posicao+8);
             env_lcd_d(' ');
             delay_200ms();
             env_lcd_c(posicao);
             env_lcd_d(j);
             env_lcd_c(posicao+4);
             env_lcd_d(j);
             env_lcd_c(posicao+8);
             env_lcd_d(j);
             delay_200ms();
            }
}

void pisca_pos_diag1(unsigned int8 posicao, unsigned int8 j){//NA DIAGONAL
           env_lcd_c(0x0c);//desliga cursor
           for(a=0;a<10;a++){
             env_lcd_c(posicao);
             env_lcd_d(' ');
             env_lcd_c(posicao+5);
             env_lcd_d(' ');
             env_lcd_c(posicao+10);
             env_lcd_d(' ');
             delay_200ms();
             env_lcd_c(posicao);
             env_lcd_d(j);
             env_lcd_c(posicao+5);
             env_lcd_d(j);
             env_lcd_c(posicao+10);
             env_lcd_d(j);
             delay_200ms();
            }
}
void pisca_pos_diag3(unsigned int8 posicao, unsigned int8 j){//NA OUTRA DIAGONAL
           env_lcd_c(0x0c);//desliga cursor
           for(a=0;a<10;a++){
             env_lcd_c(posicao);
             env_lcd_d(' ');
             env_lcd_c(posicao+3);
             env_lcd_d(' ');
             env_lcd_c(posicao+6);
             env_lcd_d(' ');
             delay_200ms();
             env_lcd_c(posicao);
             env_lcd_d(j);
             env_lcd_c(posicao+3);
             env_lcd_d(j);
             env_lcd_c(posicao+6);
             env_lcd_d(j);
             delay_200ms();
            }
}
/*******************************************************************************
             ROTINAS DE INCREMENTOS DE PONTOS PARA OS JOGADORES
*******************************************************************************/

void inc_ponto_X(unsigned int8 p){// JOGADOR 1 ('X')
  delay_200ms();//atraso de 200ms
  jogador[1]+=p;
  if(jogador[1]>9){jogador[1]-=10;;if(++jogador[0]>9){jogador[0]=0;jogador[1]=0;}}
  env_lcd_c(0xc4);
  env_lcd_d(jogador[0]|'0');//converte para ascii e envia para lcd
  env_lcd_d(jogador[1]|'0');//converte para ascii e envia para lcd
  win=1;
 }

void inc_ponto_O(unsigned int8 p){// JOGADOR 2 ('0')
  delay_200ms();//atraso de 200ms
  jogador[3]+=p;
  if(jogador[3]>9){jogador[3]-=10;if(++jogador[2]>9){jogador[2]=0;jogador[3]=0;}}
  env_lcd_c(0xcd);
  env_lcd_d(jogador[2]|'0');//converte para ascii e envia para lcd
  env_lcd_d(jogador[3]|'0');//converte para ascii e envia para lcd
  win=1;
 }

/*******************************************************************************
                           ROTINA MAIN
*******************************************************************************/
void main() {
   setup_adc_ports(NO_ANALOGS);//desliga adc
   setup_comparator(NC_NC_NC_NC);//desliga comparadores analgicos
   trisio=0b001011;//ajusta entradas e sadas
   gpio=0;//zera tudo
   descarregar();//descarga dos capacitores do Lariosplex
   Lcd_Init();//inicializa Lcd
   for(a=0;a<16;a++)env_lcd_d(read_eeprom(a));//nome do aparelho na eeprom
   delay_1seg();//mostra mensagem no lcd por um segundo
   vez=tmr0&1;
   nova_partida();//inicia tela para novo jogo
   pos=1;
   wpu0=1;//pull up habilitado para pino 6 e 7
   wpu1=1;
   gppu=0;//chave geral do pull up
   tr1=0;//flag de bloqueio da vez dos jogadores


/*******************************************************************************
                           ROTINA DE REPETIO
*******************************************************************************/
   for(;;){
     if(!cursor){//tecla de escolha de posio a ser jogada
         delay_dbc();
         tr1=1;
         pos++;//incrementa ponteiro do cursor
         cnt=20;
         while(jogo[pos]!='_'){//se for diferente deste caracter pula
           pos++;//incrementa
           cnt--;
           if(!cnt){pos=0; tr1=0;break;}//verifica se todas casas foram preenchidas
           if(pos>13){pos=2;}//se chegou no final vsivel da linha 1
         }
         if(!pos){env_lcd_c(0x90); pos=1;}
         else{
         env_lcd_c(0x80+pos);
         }
         while(!cursor)delay_dbc();
         env_lcd_c(0x0f);//liga cursor piscando
     }


      if(!vez){//verifica de quem  a vez de jogar

         if(!j1 && tr1){//se for a vez do jogador 1
           delay_dbc();//debounce do interruptor
           if(!j1){//testa de novo
           jogo[pos]='X';//altera na memria
           env_lcd_d(jogo[pos]);//mostra no display
           pos=0;
           tr1=0;
           cont++;//soma 1 ao contador de nmero de jogadas
           vez=1;//muda a sinalizao do prximo jogador
           env_lcd_c(0xc0);//mostra no lcd esta indicao
           env_lcd_d(' ');//apaga sinal no j1
           env_lcd_c(0xc9);
           env_lcd_d('>');//sinaliza j2
           env_lcd_c(0x0c);//desliga cursor
           while(!j1)delay_dbc();//delay para soltar boto
           }
        }
      }
      else{

         if(!j2 && tr1){//se for a vez do jogador 1
           delay_dbc();//debounce
           if(!j2){//testa de novo
           jogo[pos]='O';//altera na memria
           env_lcd_d(jogo[pos]);//mostra no display
           pos=0;//zera cursor
           tr1=0;
           cont++;//soma 1 ao contador de nmero de jogadas
           vez=0;//muda a sinalizao do prximo jogador
           env_lcd_c(0xc0);
           env_lcd_d('>');//sinaliza j1
           env_lcd_c(0xc9);
           env_lcd_d(' ');//apaga sinal em j2
           env_lcd_c(0x0c);//desliga cursor
           while(!j2)delay_dbc();;
           }
        }
      }


/*******************************************************************************
          ROTINA DE TESTE DE COMBINAES VITORIOSAS ( 8 POSSIBILIDADES)
          E EMPATE AO PREENCHER AS 9 POSIES DO JOGO
*******************************************************************************/

          #define PONTOS_VITORIA    3
          #define PONTOS_EMPATE     1

          res=jogo[2]+jogo[3]+jogo[4]; //3 primeiros da horizontal

          if(res==0x08) {//'x' ganhou  ("X'+'X'+'X'=0X108 ->0X08)
          pisca_pos_h(0x82,'X');
          inc_ponto_X(PONTOS_VITORIA);
          }
          if(res==0xed) {//'O' ganhou  ('O'+'O'+'O'=0XED)
          pisca_pos_h(0x82,'O');
          inc_ponto_O(PONTOS_VITORIA);
          }

          res=jogo[6]+jogo[7]+jogo[8];//3 do meio da horizontal

          if(res==0x08) {//'x' ganhou
          pisca_pos_h(0x86,'X');
          inc_ponto_X(PONTOS_VITORIA);
          }
          if(res==0xed) {//'O' ganhou
          pisca_pos_h(0x86,'O');
          inc_ponto_O(PONTOS_VITORIA);
          }

          res=jogo[10]+jogo[11]+jogo[12]; // 3 ultimos da horizontal

          if(res==0x08) {//'x' ganhou
          pisca_pos_h(0x8a,'X');
          inc_ponto_X(PONTOS_VITORIA);
          }
          if(res==0xed) {//'O' ganhou
          pisca_pos_h(0x8a,'O');
          inc_ponto_O(PONTOS_VITORIA);
          }
          //-------------------------------------------------------------

           res=jogo[2]+jogo[6]+jogo[10]; //3 primeiros da vertical

          if(res==0x08) {//'x' ganhou
          pisca_pos_v(0x82,'X');
          inc_ponto_X(PONTOS_VITORIA);
          }
          if(res==0xed) {//'O' ganhou
          pisca_pos_v(0x82,'O');
          inc_ponto_O(PONTOS_VITORIA);
          }
          res=jogo[3]+jogo[7]+jogo[11]; //3 do meio da vertical

          if(res==0x08) {//'x' ganhou
          pisca_pos_v(0x83,'X');
          inc_ponto_X(PONTOS_VITORIA);
          }
          if(res==0xed) {//'O' ganhou
          pisca_pos_v(0x83,'O');
          inc_ponto_O(PONTOS_VITORIA);
          }

          res=jogo[4]+jogo[8]+jogo[12]; //3 ultimos da vertical

          if(res==0x08) {//'x' ganhou
          pisca_pos_v(0x84,'X');
          inc_ponto_X(PONTOS_VITORIA);
          }
          if(res==0xed) {//'O' ganhou
          pisca_pos_v(0x84,'O');
          inc_ponto_O(PONTOS_VITORIA);
          }


          
          res=jogo[2]+jogo[7]+jogo[12]; //diagonal

          if(res==0x08) {//'x' ganhou
          pisca_pos_diag1(0x82,'X');
          inc_ponto_X(PONTOS_VITORIA);
          }
          if(res==0xed) {//'O' ganhou
          pisca_pos_diag1(0x82,'O');
          inc_ponto_O(PONTOS_VITORIA);
          }

          res=jogo[4]+jogo[7]+jogo[10]; //diagonal

          if(res==0x08) {//'x' ganhou
          pisca_pos_diag3(0x84,'X');
          inc_ponto_X(PONTOS_VITORIA);

          }
          if(res==0xed) {//'O' ganhou
          pisca_pos_diag3(0x84,'O');
          inc_ponto_O(PONTOS_VITORIA);
          }

          if(!win && (cont>8)){//Testa por empate
             env_lcd_c(0xc7);
             env_lcd_d('E');//sinaliza empate no Lcd 
             inc_ponto_O(PONTOS_EMPATE);//aumenta um ponto para cada jogador
             inc_ponto_X(PONTOS_EMPATE);
          }
        
          
         if(win){//aguarda apertar boto para nova partida
            while(cursor)delay_dbc();
            nova_partida();
         }
         


    }//for(;;)
}//main()




/*******************************************************************************
                           FIM DO PROGRAMA
********************************************************************************
 UFAAAAA !!!! FINALMENTE COUBE TODA ESTA 'BAGUNA' DENTRO DOS 1KBYTE DA FLASH!
*******************************************************************************/
